# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 03.11.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 10, task 5.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation

import os

import numpy as np
import cv2
import matplotlib.pyplot as plt
from scipy import ndimage # pylint: disable=import-error
#import seaborn as sns # pylint: disable=import-error,unused-import

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  fig = plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def main():
  """main"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 10')
  print('Task 5')
  print('-'*80)

  fig_num = 0
  image_dir = '../../images'
  write_dir = 'results/images'

  pcb_image = cv2.imread(os.path.join(image_dir, 'pcb.jpg'), cv2.IMREAD_GRAYSCALE)
  pcb_image = (pcb_image >= 128).astype(np.uint8)
  fig_num = plot_image(pcb_image, fig_num,
                       write_file=os.path.join(write_dir, 'pcb_orig.png'))

  pcb_filled = ndimage.binary_fill_holes(pcb_image).astype(np.uint8)
  fig_num = plot_image(pcb_filled, fig_num,
                       write_file=os.path.join(write_dir, 'pcb_filled.png'))

  pcb_holes = pcb_filled - pcb_image
  fig_num = plot_image(pcb_holes, fig_num,
                       write_file=os.path.join(write_dir, 'pcb_holes.png'))

  _, pcb_labeled, stats, centroids = cv2.connectedComponentsWithStats(pcb_holes)
  area = stats[1:, 4]
  centroids = centroids[1:, :]

  fig = plt.figure(fig_num)
  height, width = pcb_holes.shape
  plt.imshow(pcb_labeled, cmap='magma', interpolation='none')
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()

  for obj_ind, obj_area in enumerate(area):
    diameter = 2*np.sqrt(obj_area / np.pi) # Assuming circular objects
    diam_str = '{:.2f}'.format(diameter)
    plt.text(centroids[obj_ind, 0], centroids[obj_ind, 1], diam_str,
             color='green', fontsize=14, weight='bold')

    plt.plot(centroids[obj_ind, 0], centroids[obj_ind, 1], 'ro')
  axes = plt.gca()
  axes.set_xlim([0, width-1])
  axes.set_ylim([height-1, 0])
  fig.savefig(os.path.join(write_dir, 'pcb_labeled.png'), bbox_inches='tight', pad_inches=0)


  plt.show()


if __name__ == "__main__":
  main()
